/*
 * VarStar - Search & report variable star records in AAVSO data file
 * Copyright (C) 2005-2007  Frederick (Rick) Burton
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

// Star variables and function prototypes
//
// Loads vallongtype.txt into memory allowing searching by multiple
// criteria. This file is a public domain file downloaded from
// AAVSO at http://www.aavso.org

#ifndef FUNCS_H
#define FUNCS_H

#include <stdio.h>

#define FALSE 0
#define TRUE 1

// Star record node
/*static*/ struct VarStarRecord
{
	int       recnum;
	char      record[81];
	short int ra_hours;
	short int ra_minutes;
	short int dec_degrees;
	char      name[12];
	char      type[9];
	char      range[15];
	char      desc[21];
	double    max_mag;
	double    min_mag;
	double    period_days;

	struct VarStarRecord* pNext;
};

// Search result node
/*static*/ struct VarStarResult
{
	int recnum;

	struct VarStarResult* pPrev;
	struct VarStarRecord* pVSR;
	struct VarStarResult* pNext;
};

//--== Variables ==------------------------------------------------------------

// Linked list pointers to record nodes
static struct VarStarRecord* topPtr;
static struct VarStarRecord* lastPtr;
// Linked list pointers to result nodes
static struct VarStarResult* topResPtr;
static struct VarStarResult* resPtr;
static struct VarStarResult* lastResPtr;
// File pointers
static FILE* ifptr;
static FILE* ofptr;
// "State" vars
static int is_loaded;
static int is_searched;
static int is_specific;
static int begin;
static int end;
// Search criteria vars
static short int minHr;
static short int maxHr;
static short int minDeg;
static short int maxDeg;
static double minDays;
static double maxDays;
static float minMag;
static float maxMag;
// File name strings
static char filename[256]; // Data file name
static char rptname[256];  // Output file name

//--== Prototypes ==-----------------------------------------------------------

// Sets default values
// Call this function FIRST before loading records
extern void initRecords();

// Functions Setting Search Criteria Vars
extern void setSchMinHr  (short int min_ra_hrs);
extern void setSchMaxHr  (short int max_ra_hrs);
extern void setSchMinDeg (short int min_dec_deg);
extern void setSchMaxDeg (short int max_dec_deg);
extern void setSchMinDays(double min_days);
extern void setSchMaxDays(double max_days);
extern void setSchMinMag (double min_mag);
extern void setSchMaxMag (double max_mag);

// Functions Getting Search Criteria Vals
extern short int getSchMinHr  ();
extern short int getSchMaxHr  ();
extern short int getSchMinDeg ();
extern short int getSchMaxDeg ();
extern void getSchMinDays(char* mind);
extern void getSchMaxDays(char* maxd);
extern void getSchMinMag (char* minm);
extern void getSchMaxMag (char* maxm);

// Read data file, validate and load records
extern int loadRecords(const char* fname);
// Save search results as .csv
extern int saveReport (const char* fname);
// Save search results as .txt
extern int saveFile   (const char* fname);
// Test for valid record format at load
extern int isLineValidRecord(const char* testStr);

// Functions setting record node vals
extern void setHours  (struct VarStarRecord* vPtr);
extern void setMinutes(struct VarStarRecord* vPtr);
extern void setDegrees(struct VarStarRecord* vPtr);
extern void setName   (struct VarStarRecord* vPtr);
extern void setType   (struct VarStarRecord* vPtr);
extern void setRange  (struct VarStarRecord* vPtr);
extern void setMaxMag (struct VarStarRecord* vPtr);
extern void setMinMag (struct VarStarRecord* vPtr);
extern void setDays   (struct VarStarRecord* vPtr);
extern void setDesc   (struct VarStarRecord* vPtr);

// May use these funcs rather than #include <ctype.h>
//extern int isdit(char chr);
//extern int isgph(char chr);

// Clear current search results, create all-inclusive results
extern void clearSearch();
// Search based on location criteria, and when specific, variation criteria also
extern int searchRecords(int specific, int magnitude, int period);
extern int isRecordInSearch(struct VarStarRecord* vPtr, 
		int magnitude, int period);         // specific search
extern int isRecordInSearchUncertain(struct VarStarRecord* vPtr); // non-specific search

// Functions returning field values
extern short int   getHour  ();
extern short int   getMin   ();
extern short int   getDeg   ();
extern const char* getName  ();
extern const char* getType  ();
extern const char* getRange ();
extern void        getDays  (char* per);
extern const char* getDesc  ();
extern const char* getRecord();

// Record navigation 
extern void moveFirst();
extern void movePrev();
extern void moveNext();
extern void moveLast();
extern int bof();
extern int eof();

// Functions to return loaded and searched states
extern int isLoaded(); 
extern int isSearched();

// Memory clean-up funcs
extern void delRecords();
extern void delResults();

#endif
